@@ -124,7 +124,7 @@ class Key(Op):
     def values(self, node):
         return ( v for _,v in self.items(node) )
     def is_empty(self, node):
-        return not node
+        return not tuple(self.keys(node))
     def default_keys(self):
         if self.is_pattern():
             return ()
@@ -135,7 +135,7 @@ class Key(Op):
         return {self.op.value: None}
     def match(self, op):
         m = self.op.match_op(op.op)
-        return op if m else None
+        return m
     def add(self, node, key, val):
         node[key] = val
         return node
@@ -284,7 +284,7 @@ class Slice(Op):
     def match(self, op):
         if not isinstance(op, Slice):
             return None
-        return op if self.slice == op.slice else None
+        return Match(op.slice) if self.slice == op.slice else None
     def add(self, node, key, val):
         node += val if isinstance(node, (str, bytes)) else node.__class__([val])
         return node
@@ -298,7 +298,18 @@ class Slice(Op):
 #
 #
 class Dotted:
-    registry = {}
+    _registry = {}
+
+    # this is a placeholder for dynamic docstring generation only
+    @property
+    def registry(cls):
+        pass
+
+    @classmethod
+    def register(cls, name, fn):
+        cls._registry[name] = fn
+        cls.registry.__doc__ = (cls.registry.__doc__ or '') + f'\n{name}\t{fn.__doc__}'
+
     def __init__(self, results):
         self.ops = tuple(results['ops'])
         self.transforms = tuple( tuple(r) for r in results.get('transforms', ()) )
@@ -316,7 +327,7 @@ class Dotted:
         return self.ops[key]
     def apply(self, val):
         for name,*args in self.transforms:
-            fn = self.registry[name]
+            fn = self._registry[name]
             val = fn(val, *args)
         return val
 
@@ -326,10 +337,11 @@ def transform(name):
     Transform decorator
     """
     def _fn(fn):
-        Dotted.registry[name] = fn
+        Dotted.register(name, fn)
         return fn
     return _fn
 
+
 def build_default(ops):
     cur, *ops = ops
     built = cur.default()
@@ -339,6 +351,7 @@ def build_default(ops):
         built = cur.add(built, k, build_default(ops))
     return built
 
+
 def build(ops, node, deepcopy=True):
     cur, *ops = ops
     built = node.__class__()
@@ -351,6 +364,7 @@ def build(ops, node, deepcopy=True):
         built = build_default([cur]+ops)
     return built
 
+
 def gets(ops, node):
     cur, *ops = ops
     values = cur.values(node)
@@ -360,6 +374,7 @@ def gets(ops, node):
     for v in values:
         yield from gets(ops, v)
 
+
 def updates(ops, node, val):
     def _updates(ops, node, val, has_defaults=False):
         cur, *ops = ops
@@ -373,6 +388,7 @@ def updates(ops, node, val):
         return node
     return _updates(ops, node, val)
 
+
 def removes(ops, node):
     cur, *ops = ops
     if not ops:
@@ -382,6 +398,7 @@ def removes(ops, node):
         removes(ops, v)
     return node
 
+
 def expands(ops, node):
     def _expands(ops, node):
         cur, *ops = ops
